@@ -109,7 +109,10 @@ public final class AddAnnotations
             PDAnnotationTextMarkup txtMark = new PDAnnotationTextMarkup(
                     PDAnnotationTextMarkup.SUB_TYPE_HIGHLIGHT);
             txtMark.setColor(blue);
-            txtMark.setConstantOpacity((float)0.2); // 20% transparent
+            
+            // remove line below if PDF/A-2b (and possibly other PDF-A flavours)
+            // also add txtMark.setPrinted(true)
+            txtMark.setConstantOpacity((float) 0.2); // 20% transparent
 
             // Set the rectangle containing the markup
             float textWidth = font.getStringWidth("PDFBox") / 1000 * 18;
@@ -303,9 +306,11 @@ public final class AddAnnotations
             // Create the appearance streams.
             // Adobe Reader will always display annotations without appearance streams nicely,
             // but other applications may not.
+            // Pass the PDDocument so that the appearance handler can look into the default resources
+            // for non-standard fonts.
             for (PDAnnotation ann : annotations)
             {
-                ann.constructAppearances();
+                ann.constructAppearances(document);
             }
 
             showPageNo(document, page1, "Page 1");
