@@ -110,11 +110,11 @@ public class CustomPageDrawer
          * Glyph bounding boxes.
          */
         @Override
-        protected void showGlyph(Matrix textRenderingMatrix, PDFont font, int code, String unicode,
-                                 Vector displacement) throws IOException
+        protected void showGlyph(Matrix textRenderingMatrix, PDFont font, int code,
+                Vector displacement) throws IOException
         {
             // draw glyph
-            super.showGlyph(textRenderingMatrix, font, code, unicode, displacement);
+            super.showGlyph(textRenderingMatrix, font, code, displacement);
             
             // bbox in EM -> user units
             Shape bbox = new Rectangle2D.Float(0, 0, font.getWidth(code) / 1000, 1);
