@@ -86,7 +86,9 @@ public class OcspHelper
     private DEROctetString encodedNonce;
     private X509Certificate ocspResponderCertificate;
     private final JcaX509CertificateConverter certificateConverter = new JcaX509CertificateConverter();
-    private static final Random rand = new SecureRandom();
+
+    // SecureRandom.getInstanceStrong() would be better, but sometimes blocks on Linux
+    private static final Random RANDOM = new SecureRandom();
 
     /**
      * @param checkCertificate Certificate to be OCSP-checked
@@ -585,9 +587,8 @@ public class OcspHelper
 
     private byte[] create16BytesNonce()
     {
-        // replace with SecureRandom.getInstanceStrong() on jdk8 and higher
         byte[] nonce = new byte[16];
-        rand.nextBytes(nonce);
+        RANDOM.nextBytes(nonce);
         return nonce;
     }
 
