@@ -58,6 +58,11 @@ import org.bouncycastle.asn1.ocsp.OCSPObjectIdentifiers;
 import org.bouncycastle.cert.ocsp.BasicOCSPResp;
 import org.bouncycastle.cert.ocsp.OCSPException;
 import org.bouncycastle.cert.ocsp.OCSPResp;
+import org.bouncycastle.cms.CMSException;
+import org.bouncycastle.cms.CMSSignedData;
+import org.bouncycastle.tsp.TSPException;
+import org.bouncycastle.tsp.TimeStampToken;
+import org.bouncycastle.tsp.TimeStampTokenInfo;
 
 /**
  * An example for adding Validation Information to a signed PDF, inspired by ETSI TS 102 778-4
@@ -99,11 +104,27 @@ public class AddValidationInformation
     {
         if (inFile == null || !inFile.exists())
         {
-            throw new FileNotFoundException("Document for signing does not exist");
+            String err = "Document for signing ";
+            if (null == inFile)
+            {
+                err += "is null";
+            }
+            else
+            {
+                err += "does not exist: " + inFile.getAbsolutePath();
+            }
+            throw new FileNotFoundException(err);
         }
 
         PDDocument doc = PDDocument.load(inFile);
         FileOutputStream fos = new FileOutputStream(outFile);
+        int accessPermissions = SigUtils.getMDPPermission(doc);
+        if (accessPermissions == 1)
+        {
+            System.out.println("PDF is certified to forbid changes, "
+                        + "some readers may report the document as invalid despite that "
+                        + "the PDF specification allows DSS additions");
+        }
         document = doc;
         doValidation(inFile.getAbsolutePath(), fos);
         fos.close();
@@ -114,7 +135,6 @@ public class AddValidationInformation
      * Fetches certificate information from the last signature of the document and appends a DSS
      * with the validation information to the document.
      *
-     * @param document containing the Signature
      * @param filename in file to extract signature
      * @param output where to write the changed document
      * @throws IOException
@@ -130,12 +150,28 @@ public class AddValidationInformation
             {
                 certInfo = certInformationHelper.getLastCertInfo(signature, filename);
                 signDate = signature.getSignDate();
+                if ("ETSI.RFC3161".equals(signature.getSubFilter()))
+                {
+                    byte[] contents = signature.getContents();
+                    TimeStampToken timeStampToken = new TimeStampToken(new CMSSignedData(contents));
+                    TimeStampTokenInfo timeStampInfo = timeStampToken.getTimeStampInfo();
+                    signDate = Calendar.getInstance();
+                    signDate.setTime(timeStampInfo.getGenTime());
+                }
             }
         }
         catch (CertificateProccessingException e)
         {
             throw new IOException("An Error occurred processing the Signature", e);
         }
+        catch (CMSException e)
+        {
+            throw new IOException("An Error occurred processing the Signature", e);
+        }
+        catch (TSPException e)
+        {
+            throw new IOException("An Error occurred processing the Signature", e);
+        }
         if (certInfo == null)
         {
             throw new IOException(
@@ -315,17 +351,17 @@ public class AddValidationInformation
         }
         catch (OCSPException e)
         {
-            LOG.warn("Failed fetching Ocsp", e);
+            LOG.error("Failed fetching OCSP at " + certInfo.getOcspUrl(), e);
             return false;
         }
         catch (CertificateProccessingException e)
         {
-            LOG.warn("Failed fetching Ocsp", e);
+            LOG.error("Failed fetching OCSP at " + certInfo.getOcspUrl(), e);
             return false;
         }
         catch (IOException e)
         {
-            LOG.warn("Failed fetching Ocsp", e);
+            LOG.error("Failed fetching OCSP at " + certInfo.getOcspUrl(), e);
             return false;
         }
         catch (RevokedCertificateException e)
@@ -620,6 +656,7 @@ public class AddValidationInformation
 
     public static void main(String[] args) throws IOException, GeneralSecurityException
     {
+//args = new String[]{"C:\\Users\\Tilman\\Downloads\\SO67171648.pdf"};
         if (args.length != 1)
         {
             usage();
@@ -636,7 +673,7 @@ public class AddValidationInformation
         String name = inFile.getName();
         String substring = name.substring(0, name.lastIndexOf('.'));
 
-        File outFile = new File(inFile.getParent(), substring + "_ocsp.pdf");
+        File outFile = new File(inFile.getParent(), substring + "_LTV.pdf");
         addOcspInformation.validateSignature(inFile, outFile);
     }
 
