@@ -140,25 +140,28 @@ public class CMapParser
                     {
                         parseUsecmap((LiteralName) previousToken, result);
                     }
-                    else if (op.op.equals("begincodespacerange") && previousToken instanceof Number)
+                    else if (previousToken instanceof Number)
                     {
-                        parseBegincodespacerange((Number) previousToken, cmapStream, result);
-                    }
-                    else if (op.op.equals("beginbfchar") && previousToken instanceof Number)
-                    {
-                        parseBeginbfchar((Number) previousToken, cmapStream, result);
-                    }
-                    else if (op.op.equals("beginbfrange") && previousToken instanceof Number)
-                    {
-                        parseBeginbfrange((Number) previousToken, cmapStream, result);
-                    }
-                    else if (op.op.equals("begincidchar") && previousToken instanceof Number)
-                    {
-                        parseBegincidchar((Number) previousToken, cmapStream, result);
-                    }
-                    else if (op.op.equals("begincidrange") && previousToken instanceof Integer)
-                    {
-                        parseBegincidrange((Integer) previousToken, cmapStream, result);
+                        if (op.op.equals("begincodespacerange"))
+                        {
+                            parseBegincodespacerange((Number) previousToken, cmapStream, result);
+                        }
+                        else if (op.op.equals("beginbfchar"))
+                        {
+                            parseBeginbfchar((Number) previousToken, cmapStream, result);
+                        }
+                        else if (op.op.equals("beginbfrange"))
+                        {
+                            parseBeginbfrange((Number) previousToken, cmapStream, result);
+                        }
+                        else if (op.op.equals("begincidchar"))
+                        {
+                            parseBegincidchar((Number) previousToken, cmapStream, result);
+                        }
+                        else if (op.op.equals("begincidrange") && previousToken instanceof Integer)
+                        {
+                            parseBegincidrange((Integer) previousToken, cmapStream, result);
+                        }
                     }
                 }
             }
@@ -242,6 +245,25 @@ public class CMapParser
         }
     }
 
+    /**
+     * Throws an IOException if expectedOperatorName not equals operator.op
+     *
+     * @param operator Instance of operator
+     * @param expectedOperatorName Expected name of operator
+     * @param rangeName The name of the range in which the operator is expected (without a tilde
+     * character), to be used in the exception message.
+     * 
+     * @throws IOException if expectedOperatorName not equals operator.op
+     */
+    private void checkExpectedOperator(Operator operator, String expectedOperatorName, String rangeName) throws IOException
+    {
+        if (!operator.op.equals(expectedOperatorName))
+        {
+            throw new IOException("Error : ~" + rangeName + " contains an unexpected operator : "
+                    + operator.op);
+        }
+    }
+
     private void parseBegincodespacerange(Number cosCount, PushbackInputStream cmapStream, CMap result) throws IOException
     {
         for (int j = 0; j < cosCount.intValue(); j++)
@@ -249,11 +271,7 @@ public class CMapParser
             Object nextToken = parseNextToken(cmapStream);
             if (nextToken instanceof Operator)
             {
-                if (!((Operator) nextToken).op.equals("endcodespacerange"))
-                {
-                    throw new IOException("Error : ~codespacerange contains an unexpected operator : "
-                            + ((Operator) nextToken).op);
-                }
+                checkExpectedOperator((Operator) nextToken, "endcodespacerange", "codespacerange");
                 break;
             }
             byte[] startRange = (byte[]) nextToken;
@@ -276,11 +294,7 @@ public class CMapParser
             Object nextToken = parseNextToken(cmapStream);
             if (nextToken instanceof Operator)
             {
-                if (!((Operator) nextToken).op.equals("endbfchar"))
-                {
-                    throw new IOException("Error : ~bfchar contains an unexpected operator : "
-                            + ((Operator) nextToken).op);
-                }
+                checkExpectedOperator((Operator) nextToken, "endbfchar", "bfchar");
                 break;
             }
             byte[] inputCode = (byte[]) nextToken;
@@ -310,11 +324,7 @@ public class CMapParser
             Object nextToken = parseNextToken(cmapStream);
             if (nextToken instanceof Operator)
             {
-                if (!((Operator) nextToken).op.equals("endcidrange"))
-                {
-                    throw new IOException("Error : ~cidrange contains an unexpected operator : "
-                            + ((Operator) nextToken).op);
-                }
+                checkExpectedOperator((Operator) nextToken, "endcidrange", "cidrange");
                 break;
             }
             byte[] startCode = (byte[]) nextToken;
@@ -355,11 +365,7 @@ public class CMapParser
             Object nextToken = parseNextToken(cmapStream);
             if (nextToken instanceof Operator)
             {
-                if (!((Operator) nextToken).op.equals("endcidchar"))
-                {
-                    throw new IOException("Error : ~cidchar contains an unexpected operator : "
-                            + ((Operator) nextToken).op);
-                }
+                checkExpectedOperator((Operator) nextToken, "endcidchar", "cidchar");
                 break;
             }
             byte[] inputCode = (byte[]) nextToken;
@@ -376,11 +382,7 @@ public class CMapParser
             Object nextToken = parseNextToken(cmapStream);
             if (nextToken instanceof Operator)
             {
-                if (!((Operator) nextToken).op.equals("endbfrange"))
-                {
-                    throw new IOException("Error : ~bfrange contains an unexpected operator : "
-                            + ((Operator) nextToken).op);
-                }
+                checkExpectedOperator((Operator) nextToken, "endbfrange", "bfrange");
                 break;
             }
             byte[] startCode = (byte[]) nextToken;
@@ -470,7 +472,12 @@ public class CMapParser
      */
     protected InputStream getExternalCMap(String name) throws IOException
     {
-        return new BufferedInputStream(getClass().getResourceAsStream(name));
+        InputStream resourceAsStream = getClass().getResourceAsStream(name);
+        if (resourceAsStream == null)
+        {
+            throw new IOException("Error: Could not find referenced cmap stream " + name);
+        }
+        return new BufferedInputStream(resourceAsStream);
     }
 
     private Object parseNextToken(PushbackInputStream is) throws IOException
