@@ -223,7 +223,8 @@ public class CmapSubtable implements CmapLookup
         if (startCode < 0 || startCode > 0x0010FFFF || (startCode + numChars) > 0x0010FFFF
                 || ((startCode + numChars) >= 0x0000D800 && (startCode + numChars) <= 0x0000DFFF))
         {
-            throw new IOException("Invalid Characters codes");
+            throw new IOException("Invalid character codes, " + 
+                    String.format("startCode: 0x%X, numChars: %d", startCode, numChars));
 
         }
     }
@@ -237,6 +238,7 @@ public class CmapSubtable implements CmapLookup
      */
     void processSubtype12(TTFDataStream data, int numGlyphs) throws IOException
     {
+        int maxGlyphId = 0;
         long nbGroups = data.readUnsignedInt();
         glyphIdToCharacterCode = newGlyphIdToCharacterCode(numGlyphs);
         characterCodeToGlyphId = new HashMap<Integer, Integer>(numGlyphs);
@@ -254,14 +256,14 @@ public class CmapSubtable implements CmapLookup
             if (firstCode < 0 || firstCode > 0x0010FFFF ||
                 firstCode >= 0x0000D800 && firstCode <= 0x0000DFFF)
             {
-                throw new IOException("Invalid characters codes");
+                throw new IOException("Invalid character code " + String.format("0x%X", firstCode));
             }
 
             if (endCode > 0 && endCode < firstCode ||
                 endCode > 0x0010FFFF ||
                 endCode >= 0x0000D800 && endCode <= 0x0000DFFF)
             {
-                throw new IOException("Invalid characters codes");
+                throw new IOException("Invalid character code " + String.format("0x%X", endCode));
             }
 
             for (long j = 0; j <= endCode - firstCode; ++j)
@@ -278,10 +280,11 @@ public class CmapSubtable implements CmapLookup
                     LOG.warn("Format 12 cmap contains character beyond UCS-4");
                 }
 
-                glyphIdToCharacterCode[(int) glyphIndex] = (int) (firstCode + j);
+                maxGlyphId = Math.max(maxGlyphId, (int) glyphIndex);
                 characterCodeToGlyphId.put((int) (firstCode + j), (int) glyphIndex);
             }
         }
+        buildGlyphIdToCharacterCodeLookup(maxGlyphId);
     }
 
     /**
@@ -315,13 +318,13 @@ public class CmapSubtable implements CmapLookup
 
             if (firstCode < 0 || firstCode > 0x0010FFFF || (firstCode >= 0x0000D800 && firstCode <= 0x0000DFFF))
             {
-                throw new IOException("Invalid Characters codes");
+                throw new IOException("Invalid character code " + String.format("0x%X", firstCode));
             }
 
             if ((endCode > 0 && endCode < firstCode) || endCode > 0x0010FFFF
                     || (endCode >= 0x0000D800 && endCode <= 0x0000DFFF))
             {
-                throw new IOException("Invalid Characters codes");
+                throw new IOException("Invalid character code " + String.format("0x%X", endCode));
             }
 
             for (long j = 0; j <= endCode - firstCode; ++j)
