@@ -40,6 +40,17 @@ public class GlyfSimpleDescript extends GlyfDescript
     private short[] yCoordinates;
     private final int pointCount;
 
+    /**
+     * Constructor for an empty description.
+     * 
+     * @throws IOException is thrown if something went wrong
+     */
+    GlyfSimpleDescript() throws IOException
+    {
+        super((short) 0, null);
+        pointCount = 0;
+    }
+
     /**
      * Constructor.
      * 
@@ -160,7 +171,7 @@ public class GlyfSimpleDescript extends GlyfDescript
             {
                 if ((flags[i] & X_SHORT_VECTOR) != 0)
                 {
-                    x += (short) -((short) bais.readUnsignedByte());
+                    x -= (short) bais.readUnsignedByte();
                 }
                 else
                 {
@@ -183,7 +194,7 @@ public class GlyfSimpleDescript extends GlyfDescript
             {
                 if ((flags[i] & Y_SHORT_VECTOR) != 0)
                 {
-                    y += (short) -((short) bais.readUnsignedByte());
+                    y -= (short) bais.readUnsignedByte();
                 }
                 else
                 {
@@ -205,8 +216,13 @@ public class GlyfSimpleDescript extends GlyfDescript
             if ((flags[index] & REPEAT) != 0)
             {
                 int repeats = bais.readUnsignedByte();
-                for (int i = 1; i <= repeats && index + i < flags.length; i++)
+                for (int i = 1; i <= repeats; i++)
                 {
+                    if (index + i >= flags.length)
+                    {
+                        LOG.error("repeat count (" + repeats + ") higher than remaining space");
+                        return;
+                    }
                     flags[index + i] = flags[index];
                 }
                 index += repeats;
