@@ -253,7 +253,8 @@ final class Type1Parser
             }
             else
             {
-               throw new IOException("Expected INTEGER or REAL but got " + token.getKind());
+                throw new IOException("Expected INTEGER or REAL but got " + token +
+                        " at array position " + i);
             }
         }
         return numbers;
@@ -588,6 +589,7 @@ final class Type1Parser
                 read(Token.NAME, "def");
                 read(Token.END_PROC);
                 readMaybe(Token.NAME, "executeonly");
+                readMaybe(Token.NAME, "readonly");
                 read(Token.NAME, "def");
             }
             else if ("NP".equals(key))
@@ -597,6 +599,7 @@ final class Type1Parser
                 read(Token.NAME);
                 read(Token.END_PROC);
                 readMaybe(Token.NAME, "executeonly");
+                readMaybe(Token.NAME, "readonly");
                 read(Token.NAME, "def");
             }
             else if ("RD".equals(key))
@@ -606,6 +609,7 @@ final class Type1Parser
                 readProcVoid();
                 readMaybe(Token.NAME, "bind");                
                 readMaybe(Token.NAME, "executeonly");                
+                readMaybe(Token.NAME, "readonly");
                 read(Token.NAME, "def");
             }
             else
