@@ -17,6 +17,8 @@
 
 package org.apache.pdfbox.contentstream.operator.color;
 
+import org.apache.pdfbox.contentstream.operator.OperatorName;
+
 /**
  * scn: Sets the colour to use for non-stroking operations.
  * Supports Pattern, Separation, DeviceN and ICCBased colour spaces.
@@ -28,6 +30,6 @@ public class SetNonStrokingColorN extends SetNonStrokingColor
     @Override
     public String getName()
     {
-        return "scn";
+        return OperatorName.NON_STROKING_COLOR_N;
     }
 }
