@@ -21,6 +21,7 @@ import java.util.List;
 
 import org.apache.pdfbox.cos.COSBase;
 import org.apache.pdfbox.contentstream.operator.Operator;
+import org.apache.pdfbox.contentstream.operator.OperatorName;
 import org.apache.pdfbox.contentstream.operator.OperatorProcessor;
 
 /**
@@ -41,6 +42,6 @@ public class EndText extends OperatorProcessor
     @Override
     public String getName()
     {
-        return "ET";
+        return OperatorName.END_TEXT;
     }
 }
