@@ -38,6 +38,7 @@ import org.apache.pdfbox.cos.COSStream;
 import org.apache.pdfbox.io.IOUtils;
 import org.apache.pdfbox.pdmodel.PDDocument;
 import org.apache.pdfbox.pdmodel.PDPage;
+import org.apache.pdfbox.pdmodel.PDPageTree;
 import org.apache.pdfbox.pdmodel.PDResources;
 import org.apache.pdfbox.pdmodel.common.PDRectangle;
 import org.apache.pdfbox.pdmodel.graphics.form.PDFormXObject;
@@ -118,8 +119,8 @@ public class Overlay implements Closeable
                 doc = loadPDF(e.getValue());
                 loadedDocuments.put(e.getValue(), doc);
                 layouts.put(doc, getLayoutPage(doc));
+                openDocuments.add(doc);
             }
-            openDocuments.add(doc);
             specificPageOverlayPage.put(e.getKey(), layouts.get(doc));
         }
         processPages(inputPDFDocument);
@@ -273,9 +274,9 @@ public class Overlay implements Closeable
         private final PDRectangle overlayMediaBox;
         private final COSStream overlayContentStream;
         private final COSDictionary overlayResources;
-        private final int overlayRotation;
+        private final short overlayRotation;
 
-        private LayoutPage(PDRectangle mediaBox, COSStream contentStream, COSDictionary resources, int rotation)
+        private LayoutPage(PDRectangle mediaBox, COSStream contentStream, COSDictionary resources, short rotation)
         {
             overlayMediaBox = mediaBox;
             overlayContentStream = contentStream;
@@ -286,7 +287,11 @@ public class Overlay implements Closeable
 
     private LayoutPage getLayoutPage(PDDocument doc) throws IOException
     {
-        PDPage page = doc.getPage(0);
+        return createLayoutPage(doc.getPage(0));
+    }
+
+    private LayoutPage createLayoutPage(PDPage page) throws IOException
+    {
         COSBase contents = page.getCOSObject().getDictionaryObject(COSName.CONTENTS);
         PDResources resources = page.getResources();
         if (resources == null)
@@ -294,24 +299,17 @@ public class Overlay implements Closeable
             resources = new PDResources();
         }
         return new LayoutPage(page.getMediaBox(), createCombinedContentStream(contents),
-                resources.getCOSObject(), page.getRotation());
+                resources.getCOSObject(), (short) page.getRotation());
     }
     
     private Map<Integer,LayoutPage> getLayoutPages(PDDocument doc) throws IOException
     {
-        int numberOfPages = doc.getNumberOfPages();
-        Map<Integer,LayoutPage> layoutPages = new HashMap<Integer, Overlay.LayoutPage>(numberOfPages);
-        for (int i=0;i<numberOfPages;i++)
+        int i = 0;
+        Map<Integer, LayoutPage> layoutPages = new HashMap<Integer, LayoutPage>();
+        for (PDPage page : doc.getPages())
         {
-            PDPage page = doc.getPage(i);
-            COSBase contents = page.getCOSObject().getDictionaryObject(COSName.CONTENTS);
-            PDResources resources = page.getResources();
-            if (resources == null)
-            {
-                resources = new PDResources();
-            }
-            layoutPages.put(i, new LayoutPage(page.getMediaBox(), createCombinedContentStream(contents), 
-                    resources.getCOSObject(), page.getRotation()));
+            layoutPages.put(i, createLayoutPage(page));
+            i++;
         }
         return layoutPages;
     }
@@ -366,17 +364,19 @@ public class Overlay implements Closeable
     private void processPages(PDDocument document) throws IOException
     {
         int pageCounter = 0;
-        for (PDPage page : document.getPages())
+        PDPageTree pageTree = document.getPages();
+        int numberOfPages = pageTree.getCount();
+        for (PDPage page : pageTree)
         {
             pageCounter++;
-            COSDictionary pageDictionary = page.getCOSObject();
-            COSBase originalContent = pageDictionary.getDictionaryObject(COSName.CONTENTS);
-            COSArray newContentArray = new COSArray();
-            LayoutPage layoutPage = getLayoutPage(pageCounter, document.getNumberOfPages());
+            LayoutPage layoutPage = getLayoutPage(pageCounter, numberOfPages);
             if (layoutPage == null)
             {
                 continue;
             }
+            COSDictionary pageDictionary = page.getCOSObject();
+            COSBase originalContent = pageDictionary.getDictionaryObject(COSName.CONTENTS);
+            COSArray newContentArray = new COSArray();
             switch (position)
             {
                 case FOREGROUND:
