@@ -154,7 +154,7 @@ public abstract class BaseParser
         COSBase value = parseDirObject();
         skipSpaces();
         // proceed if the given object is a number and the following is a number as well
-        if (!(value instanceof COSNumber) || !isDigit())
+        if ((!(value instanceof COSNumber) || !isDigit()))
         {
             return value;
         }
@@ -213,7 +213,12 @@ public abstract class BaseParser
             }
             else if (c == '/')
             {
-                parseCOSDictionaryNameValuePair(obj);
+                // something went wrong, most likely the dictionary is corrupted
+                // stop immediately and return everything read so far
+                if (!parseCOSDictionaryNameValuePair(obj))
+                {
+                    return obj;
+                }
             }
             else
             {
@@ -277,29 +282,19 @@ public abstract class BaseParser
         return false;
     }
 
-    private void parseCOSDictionaryNameValuePair(COSDictionary obj) throws IOException
+    private boolean parseCOSDictionaryNameValuePair(COSDictionary obj) throws IOException
     {
         COSName key = parseCOSName();
         COSBase value = parseCOSDictionaryValue();
         skipSpaces();
-        if (((char) seqSource.peek()) == 'd')
-        {
-            // if the next string is 'def' then we are parsing a cmap stream
-            // and want to ignore it, otherwise throw an exception.
-            String potentialDEF = readString();
-            if (!potentialDEF.equals(DEF))
-            {
-                seqSource.unread(potentialDEF.getBytes(ISO_8859_1));
-            }
-            else
-            {
-                skipSpaces();
-            }
-        }
-
         if (value == null)
         {
             LOG.warn("Bad dictionary declaration at offset " + seqSource.getPosition());
+            return false;
+        }
+        else if (value instanceof COSInteger && !((COSInteger)value).isValid())
+        {
+            LOG.warn("Skipped out of range number value at offset " + seqSource.getPosition());
         }
         else
         {
@@ -307,6 +302,7 @@ public abstract class BaseParser
             value.setDirect(true);
             obj.setItem(key, value);
         }
+        return true;
     }
 
     protected void skipWhiteSpaces() throws IOException
@@ -660,13 +656,18 @@ public abstract class BaseParser
             else
             {
                 //it could be a bad object in the array which is just skipped
-                LOG.warn("Corrupt object reference at offset " +
-                        seqSource.getPosition() + ", start offset: " + startPosition);
-
+                LOG.warn("Corrupt array element at offset "
+                        + seqSource.getPosition() + ", start offset: " + startPosition);
+                String isThisTheEnd = readString();
+                // return immediately if a corrupt element is followed by another array
+                // to avoid a possible infinite recursion as most likely the whole array is corrupted
+                if (isThisTheEnd.isEmpty() && seqSource.peek() == '[')
+                {
+                    return po;
+                }
+                seqSource.unread(isThisTheEnd.getBytes(ISO_8859_1));
                 // This could also be an "endobj" or "endstream" which means we can assume that
                 // the array has ended.
-                String isThisTheEnd = readString();
-                seqSource.unread(isThisTheEnd.getBytes(ISO_8859_1));
                 if(ENDOBJ_STRING.equals(isThisTheEnd) || ENDSTREAM_STRING.equals(isThisTheEnd))
                 {
                     return po;
@@ -690,7 +691,7 @@ public abstract class BaseParser
     {
         return ch == ASCII_SPACE || ch == ASCII_CR || ch == ASCII_LF || ch == 9 || ch == '>' ||
                ch == '<' || ch == '[' || ch =='/' || ch ==']' || ch ==')' || ch =='(' || 
-               ch == 0 || ch == '\f';
+               ch == 0 || ch == '\f' || ch == '%';
     }
 
     /**
@@ -853,17 +854,7 @@ public abstract class BaseParser
             // check for second left bracket
             c = (char) seqSource.peek();
             seqSource.unread(leftBracket);
-            if(c == '<')
-            {
-
-                COSDictionary retval = parseCOSDictionary();
-                skipSpaces();
-                return retval;
-            }
-            else
-            {
-                return parseCOSString();
-            }
+            return c == '<' ? parseCOSDictionary() : parseCOSString();
         case '[':
             // array
             return parseCOSArray();
@@ -908,31 +899,30 @@ public abstract class BaseParser
             {
                 return parseCOSNumber();
             }
+            // This is not suppose to happen, but we will allow for it
+            // so we are more compatible with POS writers that don't
+            // follow the spec
+            long startOffset = seqSource.getPosition();
+            String badString = readString();
+            if (badString.isEmpty())
+            {
+                int peek = seqSource.peek();
+                // we can end up in an infinite loop otherwise
+                throw new IOException(
+                        "Unknown dir object c='" + c + "' cInt=" + (int) c + " peek='" + (char) peek
+                        + "' peekInt=" + peek + " at offset " + seqSource.getPosition()
+                        + " (start offset: " + startOffset + ")");
+            }
+
+            // if it's an endstream/endobj, we want to put it back so the caller will see it
+            if (ENDOBJ_STRING.equals(badString) || ENDSTREAM_STRING.equals(badString))
+            {
+                seqSource.unread(badString.getBytes(ISO_8859_1));
+            }
             else
             {
-                //This is not suppose to happen, but we will allow for it
-                //so we are more compatible with POS writers that don't
-                //follow the spec
-                String badString = readString();
-                if (badString.isEmpty())
-                {
-                    int peek = seqSource.peek();
-                    // we can end up in an infinite loop otherwise
-                    throw new IOException( "Unknown dir object c='" + c +
-                            "' cInt=" + (int)c + " peek='" + (char)peek 
-                            + "' peekInt=" + peek + " at offset " + seqSource.getPosition() );
-                }
-
-                // if it's an endstream/endobj, we want to put it back so the caller will see it
-                if(ENDOBJ_STRING.equals(badString) || ENDSTREAM_STRING.equals(badString))
-                {
-                    seqSource.unread(badString.getBytes(ISO_8859_1));
-                }
-                else
-                {
-                    LOG.warn("Skipped unexpected dir object = '" + badString + "' at offset "
-                            + seqSource.getPosition());
-                }
+                LOG.warn("Skipped unexpected dir object = '" + badString + "' at offset "
+                        + seqSource.getPosition() + " (start offset: " + startOffset + ")");
             }
         }
         return null;
