@@ -28,8 +28,10 @@ import org.apache.pdfbox.cos.COSName;
 import org.apache.pdfbox.pdmodel.common.COSObjectable;
 
 import java.util.ArrayList;
+import java.util.HashSet;
 import java.util.List;
 import java.util.NoSuchElementException;
+import java.util.Set;
 import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
 
@@ -170,10 +172,12 @@ public class PDPageTree implements COSObjectable, Iterable<PDPage>
     private final class PageIterator implements Iterator<PDPage>
     {
         private final Queue<COSDictionary> queue = new ArrayDeque<COSDictionary>();
+        private Set<COSDictionary> set = new HashSet<COSDictionary>();
 
         private PageIterator(COSDictionary node)
         {
             enqueueKids(node);
+            set = null; // release memory, we don't use this anymore
         }
 
         private void enqueueKids(COSDictionary node)
@@ -183,6 +187,16 @@ public class PDPageTree implements COSObjectable, Iterable<PDPage>
                 List<COSDictionary> kids = getKids(node);
                 for (COSDictionary kid : kids)
                 {
+                    if (set.contains(kid))
+                    {
+                        // PDFBOX-5009, PDFBOX-3953: prevent stack overflow with malformed PDFs
+                        LOG.error("This page tree node has already been visited");
+                        continue;
+                    }
+                    else if (kid.containsKey(COSName.KIDS))
+                    {
+                        set.add(kid);
+                    }
                     enqueueKids(kid);
                 }
             }
@@ -225,7 +239,9 @@ public class PDPageTree implements COSObjectable, Iterable<PDPage>
      *
      * @param index zero-based index
      * 
-     * @return the page at the given index.
+     * @throws IllegalStateException if the requested index isn't found or doesn't point to a valid
+     * page dictionary
+     * @throws IndexOutOfBoundsException if the requested index is higher than the page count
      */
     public PDPage get(int index)
     {
@@ -258,10 +274,12 @@ public class PDPageTree implements COSObjectable, Iterable<PDPage>
      * @param node page tree node to search
      * @param encountered number of pages encountered so far
      * @return COS dictionary of the Page object
+     * @throws IllegalStateException if the requested page number isn't found
+     * @throws IndexOutOfBoundsException if the requested page number is higher than the page count
      */
     private COSDictionary get(int pageNum, COSDictionary node, int encountered)
     {
-        if (pageNum < 0)
+        if (pageNum < 1)
         {
             throw new IndexOutOfBoundsException("Index out of bounds: " + pageNum);
         }
