@@ -74,7 +74,7 @@ public class PDComplexFileSpecification extends PDFileSpecification
     {
         if (efDictionary == null && fs != null)
         {
-            efDictionary = (COSDictionary)fs.getDictionaryObject( COSName.EF );            
+            efDictionary = fs.getCOSDictionary(COSName.EF);
         }
         return efDictionary;
     }
@@ -260,13 +260,12 @@ public class PDComplexFileSpecification extends PDFileSpecification
      */
     public PDEmbeddedFile getEmbeddedFile()
     {
-        PDEmbeddedFile file = null;
-        COSStream stream = (COSStream)getObjectFromEFDictionary(COSName.F);
-        if( stream != null )
+        COSBase base = getObjectFromEFDictionary(COSName.F);
+        if (base instanceof COSStream)
         {
-            file = new PDEmbeddedFile( stream );
+            return new PDEmbeddedFile((COSStream) base);
         }
-        return file;
+        return null;
     }
 
     /**
@@ -296,14 +295,13 @@ public class PDComplexFileSpecification extends PDFileSpecification
      * @return The embedded dos file for this file spec.
      */
     public PDEmbeddedFile getEmbeddedFileDos()
-    {
-        PDEmbeddedFile file = null;
-        COSStream stream = (COSStream)getObjectFromEFDictionary( COSName.DOS );
-        if( stream != null )
+    {        
+        COSBase base = getObjectFromEFDictionary( COSName.DOS );
+        if (base instanceof COSStream)
         {
-            file = new PDEmbeddedFile( stream );
+            return new PDEmbeddedFile((COSStream) base);
         }
-        return file;
+        return null;
     }
 
     /**
@@ -333,14 +331,13 @@ public class PDComplexFileSpecification extends PDFileSpecification
      * @return The embedded Mac file for this file spec.
      */
     public PDEmbeddedFile getEmbeddedFileMac()
-    {
-        PDEmbeddedFile file = null;
-        COSStream stream = (COSStream)getObjectFromEFDictionary( COSName.MAC );
-        if( stream != null )
+    {        
+        COSBase base = getObjectFromEFDictionary( COSName.MAC );
+        if (base instanceof COSStream)
         {
-            file = new PDEmbeddedFile( stream );
+            return new PDEmbeddedFile((COSStream) base);
         }
-        return file;
+        return null;
     }
 
     /**
@@ -370,14 +367,13 @@ public class PDComplexFileSpecification extends PDFileSpecification
      * @return The embedded file for this file spec.
      */
     public PDEmbeddedFile getEmbeddedFileUnix()
-    {
-        PDEmbeddedFile file = null;
-        COSStream stream = (COSStream)getObjectFromEFDictionary( COSName.UNIX );
-        if( stream != null )
+    {        
+        COSBase base = getObjectFromEFDictionary( COSName.UNIX );
+        if (base instanceof COSStream)
         {
-            file = new PDEmbeddedFile( stream );
+            return new PDEmbeddedFile((COSStream) base);
         }
-        return file;
+        return null;
     }
 
     /**
@@ -407,14 +403,13 @@ public class PDComplexFileSpecification extends PDFileSpecification
      * @return The embedded unicode file for this file spec.
      */
     public PDEmbeddedFile getEmbeddedFileUnicode()
-    {
-        PDEmbeddedFile file = null;
-        COSStream stream = (COSStream)getObjectFromEFDictionary( COSName.UF );
-        if( stream != null )
+    {        
+        COSBase base = getObjectFromEFDictionary( COSName.UF );
+        if (base instanceof COSStream)
         {
-            file = new PDEmbeddedFile( stream );
+            return new PDEmbeddedFile((COSStream) base);
         }
-        return file;
+        return null;
     }
 
     /**
