@@ -61,7 +61,7 @@ public abstract class SecurityHandler
 {
     private static final Log LOG = LogFactory.getLog(SecurityHandler.class);
 
-    private static final int DEFAULT_KEY_LENGTH = 40;
+    private static final short DEFAULT_KEY_LENGTH = 40;
 
     // see 7.6.2, page 58, PDF 32000-1:2008
     private static final byte[] AES_SALT = { (byte) 0x73, (byte) 0x41, (byte) 0x6c, (byte) 0x54 };
@@ -69,7 +69,7 @@ public abstract class SecurityHandler
     /**
      * The length in bits of the secret key used to encrypt the document. Will become private in 3.0.
      */
-    protected int keyLength = DEFAULT_KEY_LENGTH;
+    protected short keyLength = DEFAULT_KEY_LENGTH;
 
     /** The encryption key that will used to encrypt / decrypt. Will become private in 3.0. */
     protected byte[] encryptionKey;
@@ -77,7 +77,7 @@ public abstract class SecurityHandler
     /** The RC4 implementation used for cryptographic functions. */
     private final RC4Cipher rc4 = new RC4Cipher();
 
-    /** indicates if the Metadata have to be decrypted of not. */
+    /** Indicates if the Metadata have to be decrypted of not. */
     private boolean decryptMetadata;
 
     /** Can be used to allow stateless AES encryption */
@@ -120,7 +120,17 @@ public abstract class SecurityHandler
     {
         this.decryptMetadata = decryptMetadata;
     }
-    
+
+    /**
+     * Returns true if meta data is to be decrypted.
+     *
+     * @return True if meta data has to be decrypted.
+     */
+    public boolean isDecryptMetadata()
+    {
+        return decryptMetadata;
+    }
+
     /**
      * Set the string filter name.
      * 
@@ -381,7 +391,7 @@ public abstract class SecurityHandler
         {
             // read IV from stream
             int ivSize = (int) IOUtils.populateBuffer(data, iv);
-            if (ivSize == -1)
+            if (ivSize == 0)
             {
                 return false;
             }
@@ -427,10 +437,6 @@ public abstract class SecurityHandler
      */
     public void decrypt(COSBase obj, long objNum, long genNum) throws IOException
     {
-        if (!(obj instanceof COSString || obj instanceof COSDictionary || obj instanceof COSArray))
-        {
-            return;
-        }
         // PDFBOX-4477: only cache strings and streams, this improves speed and memory footprint
         if (obj instanceof COSString)
         {
@@ -652,7 +658,7 @@ public abstract class SecurityHandler
 
     /**
      * Getter of the property <tt>keyLength</tt>.
-     * @return  Returns the keyLength.
+     * @return Returns the key length in bits.
      */
     public int getKeyLength()
     {
@@ -662,11 +668,11 @@ public abstract class SecurityHandler
     /**
      * Setter of the property <tt>keyLength</tt>.
      *
-     * @param keyLen  The keyLength to set.
+     * @param keyLen The key length to set in bits.
      */
     public void setKeyLength(int keyLen)
     {
-        this.keyLength = keyLen;
+        this.keyLength = (short) keyLen;
     }
 
     /**
