@@ -150,7 +150,7 @@ public final class StandardSecurityHandler extends SecurityHandler
         // This is only used with security version 4 and 5.
         if (encryption.getVersion() >= 4) {
 	        setStreamFilterName(encryption.getStreamFilterName());
-	        setStringFilterName(encryption.getStreamFilterName());
+	        setStringFilterName(encryption.getStringFilterName());
         }
         setDecryptMetadata(encryption.isEncryptMetaData());
         StandardDecryptionMaterial material = (StandardDecryptionMaterial)decryptionMaterial;
@@ -164,6 +164,28 @@ public final class StandardSecurityHandler extends SecurityHandler
         int dicPermissions = encryption.getPermissions();
         int dicRevision = encryption.getRevision();
         int dicLength = encryption.getVersion() == 1 ? 5 : encryption.getLength() / 8;
+        
+        if (encryption.getVersion() == 4 || encryption.getVersion() == 5)
+        {
+            // detect whether AES encryption is used. This assumes that the encryption algo is 
+            // stored in the PDCryptFilterDictionary
+            // However, crypt filters are used only when V is 4 or 5.
+            PDCryptFilterDictionary stdCryptFilterDictionary = encryption.getStdCryptFilterDictionary();
+            if (stdCryptFilterDictionary != null)
+            {
+                COSName cryptFilterMethod = stdCryptFilterDictionary.getCryptFilterMethod();
+                if (COSName.AESV2.equals(cryptFilterMethod))
+                {
+                    dicLength = 128 / 8;
+                    setAES(true);
+                }
+                if (COSName.AESV3.equals(cryptFilterMethod))
+                {
+                    dicLength = 256 / 8;
+                    setAES(true);
+                }
+            }
+        }
 
         byte[] documentIDBytes = getDocumentIDBytes(documentIDArray);
 
@@ -244,21 +266,6 @@ public final class StandardSecurityHandler extends SecurityHandler
         {
             validatePerms(encryption, dicPermissions, encryptMetadata);
         }
-
-        if (encryption.getVersion() == 4 || encryption.getVersion() == 5)
-        {
-            // detect whether AES encryption is used. This assumes that the encryption algo is 
-            // stored in the PDCryptFilterDictionary
-            // However, crypt filters are used only when V is 4 or 5.
-            PDCryptFilterDictionary stdCryptFilterDictionary = encryption.getStdCryptFilterDictionary();
-
-            if (stdCryptFilterDictionary != null)
-            {
-                COSName cryptFilterMethod = stdCryptFilterDictionary.getCryptFilterMethod();
-                setAES(COSName.AESV2.equals(cryptFilterMethod) || 
-                       COSName.AESV3.equals(cryptFilterMethod));
-            }
-        }
     }
 
     private byte[] getDocumentIDBytes(COSArray documentIDArray)
@@ -560,6 +567,11 @@ public final class StandardSecurityHandler extends SecurityHandler
             
             byte[] oHash = new byte[32];
             byte[] oValidationSalt = new byte[8];
+            if (owner.length < 40)
+            {
+                // PDFBOX-5104
+                throw new IOException("Owner password is too short");
+            }
             System.arraycopy(owner, 0, oHash, 0, 32);
             System.arraycopy(owner, 32, oValidationSalt, 0, 8);
             
@@ -711,6 +723,10 @@ public final class StandardSecurityHandler extends SecurityHandler
 
         if (isOwnerPassword)
         {
+            if (oe == null)
+            {
+                throw new IOException("/Encrypt/OE entry is missing");
+            }
             byte[] oKeySalt = new byte[8];
             System.arraycopy(o, 40, oKeySalt, 0, 8);
 
@@ -727,6 +743,10 @@ public final class StandardSecurityHandler extends SecurityHandler
         }
         else
         {
+            if (ue == null)
+            {
+                throw new IOException("/Encrypt/UE entry is missing");
+            }
             byte[] uKeySalt = new byte[8];
             System.arraycopy(u, 40, uKeySalt, 0, 8);
 
@@ -1062,7 +1082,7 @@ public final class StandardSecurityHandler extends SecurityHandler
             byte[] k = md.digest(input);
             
             byte[] e = null;
-            for (int round = 0; round < 64 || ((int)e[e.length-1] & 0xFF) > round - 32; round++)
+            for (int round = 0; round < 64 || (e[e.length-1] & 0xFF) > round - 32; round++)
             {                
                 byte[] k1;
                 if (userKey != null && userKey.length >= 48)
