@@ -241,7 +241,7 @@ public class PDType1Font extends PDSimpleFont
                     length1 = repairLength1(bytes, length1);
                     length2 = repairLength2(bytes, length1, length2);
 
-                    if (bytes.length > 0 && (bytes[0] & 0xff) == PFB_START_MARKER)
+                    if ((bytes[0] & 0xff) == PFB_START_MARKER)
                     {
                         // some bad files embed the entire PFB, see PDFBOX-2607
                         t1 = Type1Font.createWithPFB(bytes);
@@ -394,7 +394,6 @@ public class PDType1Font extends PDSimpleFont
     @Override
     public float getHeight(int code) throws IOException
     {
-        String name = codeToName(code);
         if (getStandard14AFM() != null)
         {
             String afmName = getEncoding().getName(code);
@@ -402,6 +401,8 @@ public class PDType1Font extends PDSimpleFont
         }
         else
         {
+            String name = codeToName(code);
+
             // todo: should be scaled by font matrix
             return (float) genericFont.getPath(name).getBounds().getHeight();
         }
@@ -424,13 +425,13 @@ public class PDType1Font extends PDSimpleFont
             if (!encoding.contains(name))
             {
                 throw new IllegalArgumentException(
-                        String.format("U+%04X ('%s') is not available in this font %s encoding: %s",
+                        String.format("U+%04X ('%s') is not available in the font %s, encoding: %s",
                                 unicode, name, getName(), encoding.getEncodingName()));
             }
             if (".notdef".equals(name))
             {
                 throw new IllegalArgumentException(
-                        String.format("No glyph for U+%04X in font %s", unicode, getName()));
+                        String.format("No glyph for U+%04X in the font %s", unicode, getName()));
             }
         }
         else
@@ -438,7 +439,7 @@ public class PDType1Font extends PDSimpleFont
             if (!encoding.contains(name))
             {
                 throw new IllegalArgumentException(
-                        String.format("U+%04X ('%s') is not available in this font %s (generic: %s) encoding: %s",
+                        String.format("U+%04X ('%s') is not available in the font %s (generic: %s), encoding: %s",
                                 unicode, name, getName(), genericFont.getName(), encoding.getEncodingName()));
             }
 
@@ -447,12 +448,18 @@ public class PDType1Font extends PDSimpleFont
             if (nameInFont.equals(".notdef") || !genericFont.hasGlyph(nameInFont))
             {
                 throw new IllegalArgumentException(
-                        String.format("No glyph for U+%04X in font %s (generic: %s)", unicode, getName(), genericFont.getName()));
+                        String.format("No glyph for U+%04X in the font %s (generic: %s)", unicode, getName(), genericFont.getName()));
             }
         }
 
         Map<String, Integer> inverted = encoding.getNameToCodeMap();
         int code = inverted.get(name);
+        if (code < 0)
+        {
+            throw new IllegalArgumentException(
+                    String.format("U+%04X ('%s') is not available in the font %s (generic: %s), encoding: %s",
+                            unicode, name, getName(), genericFont.getName(), encoding.getEncodingName()));
+        }
         bytes = new byte[] { (byte)code };
         codeToBytesMap.put(unicode, bytes);
         return bytes;
