@@ -312,19 +312,18 @@ public abstract class PDAbstractAppearanceHandler implements PDAppearanceHandler
         }
         else if (PDAnnotationLine.LE_R_OPEN_ARROW.equals(style) || PDAnnotationLine.LE_R_CLOSED_ARROW.equals(style))
         {
-            drawArrow(cs, x + (0 - sign) * width, y, (0 - sign) * width * 9);
+            drawArrow(cs, x + (-sign) * width, y, (-sign) * width * 9);
         }
         else if (PDAnnotationLine.LE_SLASH.equals(style))
         {
             // the line is 18 x linewidth at an angle of 60°
-            cs.moveTo(x + (float) (Math.cos(Math.toRadians(60)) * width * 9),
-            y + (float) (Math.sin(Math.toRadians(60)) * width * 9));
-            cs.lineTo(x + (float) (Math.cos(Math.toRadians(240)) * width * 9),
-            y + (float) (Math.sin(Math.toRadians(240)) * width * 9));
+            float width9 = width * 9;
+            cs.moveTo(x + (float) (Math.cos(Math.toRadians(60)) * width9),
+                      y + (float) (Math.sin(Math.toRadians(60)) * width9));
+            cs.lineTo(x + (float) (Math.cos(Math.toRadians(240)) * width9),
+                      y + (float) (Math.sin(Math.toRadians(240)) * width9));
         }
 
-
-
         if (PDAnnotationLine.LE_R_CLOSED_ARROW.equals(style) || 
             PDAnnotationLine.LE_CLOSED_ARROW.equals(style))
         {
@@ -333,7 +332,7 @@ public abstract class PDAbstractAppearanceHandler implements PDAppearanceHandler
         cs.drawShape(width, hasStroke, 
                      // make sure to only paint a background color (/IC value) 
                      // for interior color styles, even if an /IC value is set.
-                     INTERIOR_COLOR_STYLES.contains(style) ? hasBackground : false);
+                     INTERIOR_COLOR_STYLES.contains(style) && hasBackground);
     }
 
     /**
@@ -352,9 +351,11 @@ public abstract class PDAbstractAppearanceHandler implements PDAppearanceHandler
         // cos(angle) = x position
         // sin(angle) = y position
         // this comes very close to what Adobe is doing
-        cs.moveTo(x + (float) (Math.cos(ARROW_ANGLE) * len), y + (float) (Math.sin(ARROW_ANGLE) * len));
+        float armX = x + (float) (Math.cos(ARROW_ANGLE) * len);
+        float armYdelta = (float) (Math.sin(ARROW_ANGLE) * len);
+        cs.moveTo(armX, y + armYdelta);
         cs.lineTo(x, y);
-        cs.lineTo(x + (float) (Math.cos(ARROW_ANGLE) * len), y - (float) (Math.sin(ARROW_ANGLE) * len));
+        cs.lineTo(armX, y - armYdelta);
     }
 
     /**
@@ -523,9 +524,12 @@ public abstract class PDAbstractAppearanceHandler implements PDAppearanceHandler
             annotation.setRectangle(addRectDifferences(getRectangle(), annotation.getRectDifferences()));
             // when the normal appearance stream was generated BBox and Matrix have been set to the
             // values of the original /Rect. As the /Rect was changed that needs to be adjusted too.
-            annotation.getNormalAppearanceStream().setBBox(getRectangle());
-            AffineTransform transform = AffineTransform.getTranslateInstance(-getRectangle().getLowerLeftX(), -getRectangle().getLowerLeftY());
-            annotation.getNormalAppearanceStream().setMatrix(transform);
+            PDRectangle rect = getRectangle();
+            PDAppearanceStream appearanceStream = annotation.getNormalAppearanceStream();
+            AffineTransform transform =
+                    AffineTransform.getTranslateInstance(-rect.getLowerLeftX(), -rect.getLowerLeftY());
+            appearanceStream.setBBox(rect);
+            appearanceStream.setMatrix(transform);
         }
         else
         {
