@@ -64,7 +64,8 @@ public class PDLinkAppearanceHandler extends PDAbstractAppearanceHandler
     public void generateNormalAppearance()
     {
         PDAnnotationLink annotation = (PDAnnotationLink) getAnnotation();
-        if (annotation.getRectangle() == null)
+        PDRectangle rect = annotation.getRectangle();
+        if (rect == null)
         {
             // 660402-p1-AnnotationEmptyRect.pdf has /Rect entry with 0 elements
             return;
@@ -99,7 +100,6 @@ public class PDLinkAppearanceHandler extends PDAbstractAppearanceHandler
             {
                 // QuadPoints shall be ignored if any coordinate in the array lies outside
                 // the region specified by Rect.
-                PDRectangle rect = annotation.getRectangle();
                 for (int i = 0; i < pathsArray.length / 2; ++i)
                 {
                     if (!rect.contains(pathsArray[i * 2], pathsArray[i * 2 + 1]))
@@ -128,19 +128,22 @@ public class PDLinkAppearanceHandler extends PDAbstractAppearanceHandler
                 pathsArray[7] = borderEdge.getUpperRightY();
             }
 
+            boolean underline = false;
+            if (pathsArray.length >= 8)
+            {
+                PDBorderStyleDictionary borderStyleDic = annotation.getBorderStyle();
+                if (borderStyleDic != null)
+                {
+                    underline = PDBorderStyleDictionary.STYLE_UNDERLINE.equals(borderStyleDic.getStyle());
+                }
+            }
             int of = 0;
             while (of + 7 < pathsArray.length)
             {
-                if (annotation.getBorderStyle() != null &&
-                    annotation.getBorderStyle().getStyle().equals(PDBorderStyleDictionary.STYLE_UNDERLINE))
+                contentStream.moveTo(pathsArray[of], pathsArray[of + 1]);
+                contentStream.lineTo(pathsArray[of + 2], pathsArray[of + 3]);
+                if (!underline)
                 {
-                    contentStream.moveTo(pathsArray[of], pathsArray[of + 1]);
-                    contentStream.lineTo(pathsArray[of + 2], pathsArray[of + 3]);
-                }
-                else
-                {
-                    contentStream.moveTo(pathsArray[of], pathsArray[of + 1]);
-                    contentStream.lineTo(pathsArray[of + 2], pathsArray[of + 3]);
                     contentStream.lineTo(pathsArray[of + 4], pathsArray[of + 5]);
                     contentStream.lineTo(pathsArray[of + 6], pathsArray[of + 7]);
                     contentStream.closePath();
