@@ -26,6 +26,7 @@ import org.apache.commons.logging.LogFactory;
 import org.apache.pdfbox.cos.COSName;
 import org.apache.pdfbox.io.IOUtils;
 import org.apache.pdfbox.pdmodel.PDAppearanceContentStream;
+import org.apache.pdfbox.pdmodel.PDDocument;
 import org.apache.pdfbox.pdmodel.common.PDRectangle;
 import org.apache.pdfbox.pdmodel.font.PDType1Font;
 import org.apache.pdfbox.pdmodel.graphics.blend.BlendMode;
@@ -70,6 +71,11 @@ public class PDTextAppearanceHandler extends PDAbstractAppearanceHandler
         super(annotation);
     }
 
+    public PDTextAppearanceHandler(PDAnnotation annotation, PDDocument document)
+    {
+        super(annotation, document);
+    }
+
     @Override
     public void generateAppearanceStreams()
     {
