@@ -87,7 +87,11 @@ public class PDNonTerminalField extends PDField
         
         List<FDFField> fdfKids = fdfField.getKids();
         List<PDField> children = getChildren();
-        for (int i = 0; fdfKids != null && i < fdfKids.size(); i++)
+        if (fdfKids == null)
+        {
+            return;
+        }
+        for (int i = 0; i < fdfKids.size(); i++)
         {
             for (PDField pdChild : children)
             {
@@ -109,7 +113,7 @@ public class PDNonTerminalField extends PDField
         fdfField.setValue(getValue());
 
         List<PDField> children = getChildren();
-        List<FDFField> fdfChildren = new ArrayList<FDFField>();
+        List<FDFField> fdfChildren = new ArrayList<FDFField>(children.size());
         for (PDField child : children)
         {
             fdfChildren.add(child.exportFDF());
@@ -129,7 +133,11 @@ public class PDNonTerminalField extends PDField
     public List<PDField> getChildren()
     {
         List<PDField> children = new ArrayList<PDField>();
-        COSArray kids = (COSArray)getCOSObject().getDictionaryObject(COSName.KIDS);
+        COSArray kids = getCOSObject().getCOSArray(COSName.KIDS);
+        if (kids == null)
+        {
+            return children;
+        }
         for (int i = 0; i < kids.size(); i++)
         {
             COSBase kid = kids.getObject(i);
@@ -255,7 +263,6 @@ public class PDNonTerminalField extends PDField
     @Override
     public List<PDAnnotationWidget> getWidgets()
     {
-        List<PDAnnotationWidget> emptyList = Collections.emptyList();
-        return Collections.unmodifiableList(emptyList);
+        return Collections.emptyList();
     }
 }
