@@ -40,7 +40,7 @@ import org.apache.pdfbox.contentstream.operator.markedcontent.EndMarkedContentSe
  */
 public class PDFMarkedContentExtractor extends LegacyPDFStreamEngine
 {
-    private final boolean suppressDuplicateOverlappingText = true;
+    private boolean suppressDuplicateOverlappingText = true;
     private final List<PDMarkedContent> markedContents = new ArrayList<PDMarkedContent>();
     private final Deque<PDMarkedContent> currentMarkedContents = new ArrayDeque<PDMarkedContent>();
     private final Map<String, List<TextPosition>> characterListMapping = new HashMap<String, List<TextPosition>>();
@@ -68,6 +68,28 @@ public class PDFMarkedContentExtractor extends LegacyPDFStreamEngine
         // todo: MP - Marked Content Point with Properties
     }
 
+    /**
+     * @return the suppressDuplicateOverlappingText setting.
+     */
+    public boolean isSuppressDuplicateOverlappingText()
+    {
+        return suppressDuplicateOverlappingText;
+    }
+
+    /**
+     * By default the class will attempt to remove text that overlaps each other. Word paints the
+     * same character several times in order to make it look bold. By setting this to false all text
+     * will be extracted, which means that certain sections will be duplicated, but better
+     * performance will be noticed.
+     *
+     * @param suppressDuplicateOverlappingText The suppressDuplicateOverlappingText setting to set.
+     */
+    public void setSuppressDuplicateOverlappingText(boolean suppressDuplicateOverlappingText)
+    {
+        this.suppressDuplicateOverlappingText = suppressDuplicateOverlappingText;
+    }
+
+
     /**
      * This will determine of two floating point numbers are within a specified variance.
      *
@@ -156,10 +178,9 @@ public class PDFMarkedContentExtractor extends LegacyPDFStreamEngine
             float tolerance = (text.getWidth()/textCharacter.length())/3.0f;
             for (TextPosition sameTextCharacter : sameTextCharacters)
             {
-                TextPosition character = sameTextCharacter;
-                String charCharacter = character.getUnicode();
-                float charX = character.getX();
-                float charY = character.getY();
+                String charCharacter = sameTextCharacter.getUnicode();
+                float charX = sameTextCharacter.getX();
+                float charY = sameTextCharacter.getY();
                 //only want to suppress
                 if( charCharacter != null &&
                         //charCharacter.equals( textCharacter ) &&
