@@ -338,4 +338,78 @@ public class TestPublicKeyEncryption
         document.save(file);
         return file;
     }
-}
\ No newline at end of file
+
+    @Test
+    public void testReadPubkeyEncryptedAES128() throws IOException
+    {
+        InputStream is = TestPublicKeyEncryption.class.getResourceAsStream("AESkeylength128.pdf");
+        PDDocument doc = PDDocument.load(is,
+                "w!z%C*F-JaNdRgUk",
+                TestPublicKeyEncryption.class.getResourceAsStream("PDFBOX-4421-keystore.pfx"),
+                "testnutzer");
+        PDFTextStripper stripper = new PDFTextStripper();
+        Assert.assertEquals("Key length: 128", stripper.getText(doc).trim());
+        is.close();
+        doc.close();
+    }
+
+    @Test
+    public void testReadPubkeyEncryptedAES256() throws IOException
+    {
+        InputStream is = TestPublicKeyEncryption.class.getResourceAsStream("AESkeylength256.pdf");
+        PDDocument doc = PDDocument.load(is,
+                "w!z%C*F-JaNdRgUk",
+                TestPublicKeyEncryption.class.getResourceAsStream("PDFBOX-4421-keystore.pfx"),
+                "testnutzer");
+        PDFTextStripper stripper = new PDFTextStripper();
+        Assert.assertEquals("Key length: 256", stripper.getText(doc).trim());
+        is.close();
+        doc.close();
+    }
+
+    /**
+     * PDFBOX-5249: Read a file encrypted with AES128 but not with PDFBox, and with exposed
+     * Metadata.
+     *
+     * @throws IOException
+     */
+    @Test
+    public void testReadPubkeyEncryptedAES128withMetadataExposed() throws IOException
+    {
+        InputStream is = TestPublicKeyEncryption.class.getResourceAsStream("AES128ExposedMeta.pdf");
+        PDDocument doc = PDDocument.load(is, "",
+                TestPublicKeyEncryption.class.getResourceAsStream("PDFBOX-5249.p12"), "test",
+                MemoryUsageSetting.setupMainMemoryOnly());
+        Assert.assertEquals("PublicKeySecurityHandler",
+                doc.getEncryption().getSecurityHandler().getClass().getSimpleName());
+        Assert.assertEquals(128, doc.getEncryption().getSecurityHandler().getKeyLength());
+        PDFTextStripper stripper = new PDFTextStripper();
+        stripper.setLineSeparator("\n");
+        Assert.assertEquals("AES key length: 128\nwith exposed Metadata", stripper.getText(doc).trim());
+        doc.close();
+        is.close();
+    }
+
+    /**
+     * PDFBOX-5249: Read a file encrypted with AES128 but not with PDFBox, and with exposed
+     * Metadata.
+     *
+     * @throws IOException
+     */
+    @Test
+    public void testReadPubkeyEncryptedAES256withMetadataExposed() throws IOException
+    {
+        InputStream is = TestPublicKeyEncryption.class.getResourceAsStream("AES256ExposedMeta.pdf");
+        PDDocument doc = PDDocument.load(is, "",
+                TestPublicKeyEncryption.class.getResourceAsStream("PDFBOX-5249.p12"), "test",
+                MemoryUsageSetting.setupMainMemoryOnly());
+        Assert.assertEquals("PublicKeySecurityHandler",
+                doc.getEncryption().getSecurityHandler().getClass().getSimpleName());
+        Assert.assertEquals(256, doc.getEncryption().getSecurityHandler().getKeyLength());
+        PDFTextStripper stripper = new PDFTextStripper();
+        stripper.setLineSeparator("\n");
+        Assert.assertEquals("AES key length: 256 \nwith exposed Metadata", stripper.getText(doc).trim());
+        doc.close();
+        is.close();
+    }
+}
