@@ -606,7 +606,7 @@ public class PDFMergerUtilityTest extends TestCase
             }
         }
         br.close();
-        assertEquals(1, count);
+        assertEquals(file.getPath(), 1, count);
     }
 
     /**
@@ -684,6 +684,63 @@ public class PDFMergerUtilityTest extends TestCase
         assertTrue(outFile.delete());
     }
 
+
+    /**
+     * Check that there is a top level Document and Parts below in a merge of 2 documents.
+     *
+     * @throws IOException
+     */
+    public void testPDFBox5198_2() throws IOException
+    {
+        PDFMergerUtility pdfMergerUtility = new PDFMergerUtility();
+        pdfMergerUtility.addSource(new File(SRCDIR, "PDFA3A.pdf"));
+        pdfMergerUtility.addSource(new File(SRCDIR, "PDFA3A.pdf"));
+        pdfMergerUtility.setDestinationFileName(TARGETTESTDIR + "PDFA3A-merged2.pdf");
+        pdfMergerUtility.mergeDocuments(MemoryUsageSetting.setupMainMemoryOnly());
+
+        checkParts(new File(TARGETTESTDIR + "PDFA3A-merged2.pdf"));
+    }
+    
+    /**
+     * Check that there is a top level Document and Parts below in a merge of 3 documents.
+     *
+     * @throws IOException
+     */
+    public void testPDFBox5198_3() throws IOException
+    {
+        PDFMergerUtility pdfMergerUtility = new PDFMergerUtility();
+        pdfMergerUtility.addSource(new File(SRCDIR, "PDFA3A.pdf"));
+        pdfMergerUtility.addSource(new File(SRCDIR, "PDFA3A.pdf"));
+        pdfMergerUtility.addSource(new File(SRCDIR, "PDFA3A.pdf"));
+        pdfMergerUtility.setDestinationFileName(TARGETTESTDIR + "PDFA3A-merged3.pdf");
+        pdfMergerUtility.mergeDocuments(MemoryUsageSetting.setupMainMemoryOnly());
+
+        checkParts(new File(TARGETTESTDIR + "PDFA3A-merged3.pdf"));
+    }
+
+    /**
+     * Check that there is a top level Document and Parts below.
+     * @param file
+     * @throws IOException 
+     */
+    private void checkParts(File file) throws IOException
+    {
+        PDDocument doc = PDDocument.load(file);
+        PDStructureTreeRoot structureTreeRoot = doc.getDocumentCatalog().getStructureTreeRoot();
+        COSDictionary topDict = (COSDictionary) structureTreeRoot.getK();
+        assertEquals(COSName.DOCUMENT, topDict.getItem(COSName.S));
+        assertEquals(structureTreeRoot.getCOSObject(), topDict.getCOSDictionary(COSName.P));
+        COSArray kArray = topDict.getCOSArray(COSName.K);
+        assertEquals(doc.getNumberOfPages(), kArray.size());
+        for (int i = 0; i < kArray.size(); ++i)
+        {
+            COSDictionary dict = (COSDictionary) kArray.getObject(i);
+            assertEquals(COSName.PART, dict.getItem(COSName.S));
+            assertEquals(topDict, dict.getCOSDictionary(COSName.P));
+        }
+        doc.close();
+    }
+
     private void checkForPageOrphans(PDDocument doc) throws IOException
     {
         // check for orphan pages in the StructTreeRoot/K, StructTreeRoot/ParentTree and
