@@ -51,7 +51,7 @@ import org.w3c.dom.Element;
 /**
  * This class is a simple main class used to check the validity of a pdf file.
  * 
- * Usage : java net.awl.edoc.pdfa.Validator &lt;file path&gt;
+ * Usage : java org.apache.pdfbox.preflight.Validator_A1b &lt;file path&gt;
  * 
  * @author gbailleul
  * 
@@ -68,17 +68,6 @@ public class Validator_A1b
             System.exit(1);
         }
 
-        try
-        {
-            // force KCMS (faster than LCMS) if available
-            Class.forName("sun.java2d.cmm.kcms.KcmsServiceProvider");
-            System.setProperty("sun.java2d.cmm", "sun.java2d.cmm.kcms.KcmsServiceProvider");
-        }
-        catch (ClassNotFoundException e)
-        {
-            // ignore
-        }
-
         // is output xml ?
         int posFile = 0;
         boolean outputXml = "xml".equals(args[posFile]);
@@ -106,12 +95,14 @@ public class Validator_A1b
             }
             else
             {
+                @SuppressWarnings({"squid:S4435"}) // self-created XML
                 Transformer transformer = TransformerFactory.newInstance().newTransformer();
                 transformer.setOutputProperty(OutputKeys.INDENT, "yes");
                 transformer.setOutputProperty("{http://xml.apache.org/xslt}indent-amount", "2");
                 XmlResultParser xrp = new XmlResultParser();
                 if (isGroup)
                 {
+                    @SuppressWarnings({"squid:S2755"}) // self-created XML
                     Document document = DocumentBuilderFactory.newInstance().newDocumentBuilder().newDocument();
                     Element root = document.createElement("preflights");
                     document.appendChild(root);
@@ -152,6 +143,7 @@ public class Validator_A1b
                 Element result = xrp.validate(new FileDataSource(args[posFile]));
                 Document document = result.getOwnerDocument();
                 document.appendChild(result);
+                @SuppressWarnings({"squid:S4435"}) // self-created XML
                 Transformer transformer = TransformerFactory.newInstance().newTransformer();
                 transformer.setOutputProperty(OutputKeys.INDENT, "yes");
                 transformer.setOutputProperty("{http://xml.apache.org/xslt}indent-amount", "2");
@@ -161,7 +153,7 @@ public class Validator_A1b
 
     }
 
-    private static void usage() throws IOException 
+    private static void usage()
     {
         String version = Version.getVersion();
 
@@ -226,16 +218,23 @@ public class Validator_A1b
         if (f.isFile())
         {
             FileReader fr = new FileReader(f);
-            BufferedReader buf = new BufferedReader(fr);
-            while (buf.ready())
+            BufferedReader bufferedReader = null;
+            try
             {
-                File fn = new File(buf.readLine());
-                if (fn.exists())
+                bufferedReader = new BufferedReader(fr);
+                while (bufferedReader.ready())
                 {
-                    files.add(fn);
-                } // else warn ?
+                    File fn = new File(bufferedReader.readLine());
+                    if (fn.exists())
+                    {
+                        files.add(fn);
+                    } // else warn ?
+                }
+            }
+            finally
+            {
+                IOUtils.closeQuietly(bufferedReader);
             }
-            IOUtils.closeQuietly(buf);
         }
         else
         {
