@@ -26,7 +26,6 @@ import org.apache.pdfbox.debugger.PDFDebugger;
  */
 public final class PDFBox 
 {
-
     private PDFBox()
     {
     }
@@ -35,6 +34,7 @@ public final class PDFBox
      * Main method.
      * 
      * @param args command line arguments
+     * @throws java.lang.Exception
      */
     public static void main(String[] args) throws Exception
     {
@@ -88,6 +88,10 @@ public final class PDFBox
             {
                 PDFToImage.main(arguments);
             }
+            else if (command.equals("ImageToPDF"))
+            {
+                ImageToPDF.main(arguments);
+            }
             else if (command.equals("TextToPDF"))
             {
                 TextToPDF.main(arguments);
