@@ -81,7 +81,7 @@ public final class ImageIOUtil
         String formatName = filename.substring(filename.lastIndexOf('.') + 1);
         if ("png".equalsIgnoreCase(formatName))
         {
-            // PDFBOX-4655: prevent huge PNG files on jdk11 / jdk12 / jjdk13
+            // PDFBOX-4655: prevent huge PNG files on jdk11 / jdk12 / jdk13
             compressionQuality = 0f;
         }
         return writeImage(image, filename, dpi, compressionQuality);
@@ -186,7 +186,7 @@ public final class ImageIOUtil
         float compressionQuality = 1f;
         if ("png".equalsIgnoreCase(formatName))
         {
-            // PDFBOX-4655: prevent huge PNG files on jdk11 / jdk12 / jjdk13
+            // PDFBOX-4655: prevent huge PNG files on jdk11 / jdk12 / jdk13
             compressionQuality = 0f;
         }
         return writeImage(image, formatName, output, dpi, compressionQuality);
@@ -278,12 +278,14 @@ public final class ImageIOUtil
                 LOG.error("Supported formats: " + Arrays.toString(ImageIO.getWriterFormatNames()));
                 return false;
             }
+            
+            boolean isTifFormat = formatName.toLowerCase().startsWith("tif");
 
             // compression
             if (param != null && param.canWriteCompressed())
             {
                 param.setCompressionMode(ImageWriteParam.MODE_EXPLICIT);
-                if (formatName.toLowerCase().startsWith("tif"))
+                if (isTifFormat)
                 {
                     if ("".equals(compressionType))
                     {
@@ -306,28 +308,28 @@ public final class ImageIOUtil
                 }
             }
 
-            if (formatName.toLowerCase().startsWith("tif"))
+            if (metadata != null)
             {
-                // TIFF metadata
-                TIFFUtil.updateMetadata(metadata, image, dpi);
-            }
-            else if ("jpeg".equalsIgnoreCase(formatName)
-                    || "jpg".equalsIgnoreCase(formatName))
-            {
-                // This segment must be run before other meta operations,
-                // or else "IIOInvalidTreeException: Invalid node: app0JFIF"
-                // The other (general) "meta" methods may not be used, because
-                // this will break the reading of the meta data in tests
-                JPEGUtil.updateMetadata(metadata, dpi);
-            }
-            else
-            {
-                // write metadata is possible
-                if (metadata != null
-                        && !metadata.isReadOnly()
-                        && metadata.isStandardMetadataFormatSupported())
+                if (isTifFormat)
                 {
-                    setDPI(metadata, dpi, formatName);
+                    // TIFF metadata
+                    TIFFUtil.updateMetadata(metadata, image, dpi);
+                }
+                else if ("jpeg".equalsIgnoreCase(formatName) || "jpg".equalsIgnoreCase(formatName))
+                {
+                    // This segment must be run before other meta operations,
+                    // or else "IIOInvalidTreeException: Invalid node: app0JFIF"
+                    // The other (general) "meta" methods may not be used, because
+                    // this will break the reading of the meta data in tests
+                    JPEGUtil.updateMetadata(metadata, dpi);
+                }
+                else
+                {
+                    // write metadata is possible
+                    if (!metadata.isReadOnly() && metadata.isStandardMetadataFormatSupported())
+                    {
+                        setDPI(metadata, dpi, formatName);
+                    }
                 }
             }
 
